// Jasmine custom boot.js file

(function() {

	window.BuddyTestRunner = function (nameSpace, testOptions, defineTests) {

		const jasmine = jasmineRequire.core(jasmineRequire);

		if (testOptions && testOptions.timeout !== undefined) {
			jasmine.DEFAULT_TIMEOUT_INTERVAL = testOptions.timeout;
		}

		const env = jasmine.getEnv();

		const jasmineInterface = jasmineRequire.interface(jasmine, env);

		extend(nameSpace, jasmineInterface);

		env.addReporter(jasmineInterface.jsApiReporter);

		defineTests(); // callback

		return new Promise(function (resolve, reject) {

			const logs = [];
			const print = function (text) {
				console.log(text);
				logs.push(text);
			}
			env.addReporter({
				jasmineStarted: function(suiteInfo) {
					print('BuddyTestRunner started with '
					      + suiteInfo.totalSpecsDefined
					      + ' spec'
					      + (suiteInfo.totalSpecsDefined >= 2 ? 's' : ''));
				},
				suiteStarted: function(result) {
					print('* Suite started: "' + result.fullName + '"');
				},
				specStarted: function(result) {
					//print('** Spec started: "' + result.fullName + '"');
				},
				specDone: function(result) {
					print('** Spec "'
					      + result.fullName + '" ... '
					      + result.status);
					result.failedExpectations.forEach(function (err) {
						print('** Failure: ' + err.message);
						print(err.stack);
					});
				},
				suiteDone: function(result) {
					print('* Suite "'
					      + result.fullName + '" ... '
					      + result.status);
					result.failedExpectations.forEach(function (err) {
						print('* Failure: ' + err.message);
						print(err.stack);
					});
				},
				jasmineDone: function(result) {
					print('BuddyTestRunner done: overall status ... '
					      + result.overallStatus);
					result.failedExpectations.forEach(function (err) {
						print('Failure: ' + err.message);
						print(err.stack);
					});
					resolve({
						status: result.overallStatus,
						logs: logs
					});
				}
			});
			
			env.execute(); // run the tests
		});
	};

	// Setting up timing functions to be able to be overridden.
	// Certain browsers (Safari, IE 8, phantomjs) require this hack.
	window.setTimeout = window.setTimeout;
	window.setInterval = window.setInterval;
	window.clearTimeout = window.clearTimeout;
	window.clearInterval = window.clearInterval;

	function extend(destination, source) {
		for (var property in source) destination[property] = source[property];
		return destination;
	}
}());
